import type { TextStyle } from 'react-native'

export interface HighlighterStyleSheet {
  [key: string]: TextStyle
}

export type ReactStyle = Record<string, TextStyle>

// Helper function to convert hex color to RGB
function hexToRgb(hex: string): { r: number, g: number, b: number } | null {
  // Remove # if present
  hex = hex.replace(/^#/, '')

  // Handle both 3 and 6 digit hex
  if (hex.length === 3) {
    hex = `${hex[0]}${hex[0]}${hex[1]}${hex[1]}${hex[2]}${hex[2]}`
  }

  const result = /^([a-f\d]{2})([a-f\d]{2})([a-f\d]{2})$/i.exec(hex)
  return result
    ? {
        r: Number.parseInt(result[1]!, 16),
        g: Number.parseInt(result[2]!, 16),
        b: Number.parseInt(result[3]!, 16),
      }
    : null
}

function normalizeColor(color: string | undefined): string {
  if (!color) {
    return 'transparent'
  }

  // Handle hex colors
  if (color.startsWith('#')) {
    const rgb = hexToRgb(color)
    if (rgb) {
      return `rgb(${rgb.r}, ${rgb.g}, ${rgb.b})`
    }
  }

  // Handle rgb/rgba
  if (color.startsWith('rgb')) {
    return color
  }

  // Handle named colors
  return color
}

export function convertTokenColorsToReactStyle(tokenColors: any[]): ReactStyle {
  const styleMap: ReactStyle = {}

  // Add default styles
  styleMap.default = {
    color: '#e1e4e8',
    backgroundColor: 'transparent',
  }

  tokenColors.forEach((token) => {
    if (!token.settings)
      return

    // Handle global theme settings
    if (!token.scope) {
      if (token.settings.foreground) {
        styleMap.default!.color = normalizeColor(token.settings.foreground)
      }
      if (token.settings.background) {
        styleMap.default!.backgroundColor = normalizeColor(token.settings.background)
      }
      return
    }

    const scopes = Array.isArray(token.scope) ? token.scope : [token.scope]
    const style: TextStyle = {}

    if (token.settings.foreground) {
      style.color = normalizeColor(token.settings.foreground)
    }
    if (token.settings.background) {
      style.backgroundColor = normalizeColor(token.settings.background)
    }
    if (token.settings.fontStyle) {
      if (token.settings.fontStyle.includes('italic')) {
        style.fontStyle = 'italic'
      }
      if (token.settings.fontStyle.includes('bold')) {
        style.fontWeight = 'bold'
      }
    }

    scopes.forEach((scope: string) => {
      styleMap[scope] = { ...(styleMap[scope] || {}), ...style }
    })
  })

  return styleMap
}

interface ThemeColors {
  'editor.background'?: string
  'editor.foreground'?: string
  [key: string]: string | undefined
}

interface Theme {
  colors?: ThemeColors
  tokenColors?: any[]
}

export function getRNStylesFromShikiStyle(theme: Theme): HighlighterStyleSheet {
  if (!theme)
    return {}

  try {
    // Start with default styles
    const styles: HighlighterStyleSheet = {
      editor: {
        backgroundColor: normalizeColor(theme.colors?.['editor.background']),
      },
      base: {
        color: normalizeColor(theme.colors?.['editor.foreground']),
      },
    }

    // Process token colors
    if (theme.tokenColors) {
      const tokenStyles = convertTokenColorsToReactStyle(theme.tokenColors)
      Object.assign(styles, tokenStyles)
    }

    return styles
  }
  catch {
    return {
      editor: { backgroundColor: '#1e1e1e' },
      base: { color: '#d4d4d4' },
    }
  }
}
